SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO



ALTER     Proc up_WordSearch
--The proc searches at most 100 full text unicode characters
@sWordSearch nvarchar (100),
--ixPage=0 means the latest page
@ixPage int,
@lPageUnit int
as
SET NOCOUNT ON
------------------------------------
/*
CreateDate: 		2005-12-28
UpdateDate:		2006-01-23
UpdateDate:		2006-02-19
UpdateDate:		2006-03-15
CreateBy:		DEXLEAP.COM

CreatePurpose:
search EXACT/ALL words in Latin or CJK 

NOTE:
1. Word/Phrase are different ("thank" is a word;"thank you" is phrase)
2. Use comma to break phrase.("thank you, exciting" means searching "thank you" and "exciting")
4. The result page is based on "createdate desc" sequence
5. Only search entries with "index done" status

PLAN:

1. Break phrases into phrase table, one phrase one record
2. Each phrase will lead to a staging result table 

   If there are more than 1 phrases, 
   The table will be consisted with 
   common ixArchiveEntry + ixStringChunk + ixPhrase

   If there is only 1 phrase, The table will be consisted with
   ixArchiveEntry + ixStringChunk + ixPhrase	

3. The staging result table will lead into another table
   GroupBy ixArchiveEntry + First ixStringChunk + GroupBy ixPhrase

4. Build the final result (ixArchiveEntry, sEntryTitle, sPoster, sPostDate(time),
   sContentBrief,sGroupTitle

===================

RevisDate:
ReviseBy:
ReviseReason:


*/
------------------------------------
--time stamp (Just like the stop watch to mark the search start-time),
Declare
	@dtStartTime datetime
Select
	@dtStartTime = getdate()	

------------------------------------
------------------------------------
--PART 1: Create phrase table
------------------------------------
------------------------------------

--Customize Declaration
------------------------------------
declare 
	
	@lenStringInput int,
	@sPhraseFromInput nvarchar(100),
	@sCharFromInput nvarchar(1),
	@ixLoopCharFromInput int

create table #tPhraseFromInput
(
	ixPhrase int IDENTITY (1, 1) NOT NULL, 
	--each phrase is 100 nchar long limited
	--each word is 20 nchar long limited
	sPhraseFromInput nvarchar(100)

)
------------------------------------
--Customize Initialize
------------------------------------
Select
	@sWordSearch = ltrim(rtrim(@sWordSearch))	

select 
	@sPhraseFromInput = N'',
	@sCharFromInput = N'',
	@ixLoopCharFromInput = 1,
	@lenStringInput = len(@sWordSearch)

------------------------------------
--Main Logic
------------------------------------
While(@ixLoopCharFromInput<=@lenStringInput)
Begin
	Select
		@sCharFromInput = substring(@sWordSearch,@ixLoopCharFromInput,1)
	if @sCharFromInput <> N',' and @sCharFromInput <> ','
	Begin
		Select
			@sPhraseFromInput = @sPhraseFromInput + @sCharFromInput

	End
	Else
	Begin
		Insert into #tPhraseFromInput 
			(sPhraseFromInput)
		values 
			(rtrim(ltrim(@sPhraseFromInput)))
			
		Select
			@sPhraseFromInput = N''
					
	End
	Select
		@ixLoopCharFromInput = @ixLoopCharFromInput + 1
		
End

Insert into #tPhraseFromInput 
	(sPhraseFromInput)
values 
	(rtrim(ltrim(@sPhraseFromInput)))

------------------------------------
------------------------------------
--PART 2: phrase breaks into detailed word table
------------------------------------
------------------------------------
------------------------------------
------------------------------------
--customize declaration
------------------------------------
declare 
	@ixLoopPhrase int,
	@lenPhrase int,
	@sPhrase nvarchar(100),
	@sCharToBeWord nvarchar(1),
	@sWord nvarchar(20),
	@ixLoopPhraseText int,
	@unicodeChar int

create table #tWord
(
	
	--the primary key of this table
	[ixWordSort] int IDENTITY (1, 1) NOT NULL,
	--ixPhrase-ixWordPos pair must be unique
	[ixPhrase] int,
	[ixWordPos] int,
	[ixWordIndex] int,	
	[sWord] nvarchar(20),
	--to check if the later search loop needs it or not
	[ynLoop] varchar(3) default 'yes'
)

--Every phrase's first word
create table #tWordFirstPos
(
	--link back to #tWord
	[ixWordSort] int,
	[ixPhrase] int,
	[ixWordIndex] int,
	[ixWordPos] int
)

------------------------------------
--customized initialization
------------------------------------
Select
	@ixLoopPhrase = 0,
	@lenPhrase = 0,
	@sPhrase = N'',
	@sCharToBeWord = N'',
	@sWord = N''
------------------------------------
--main logic
------------------------------------
while(1=1)
Begin
	Select
		@ixLoopPhrase = min(ixPhrase)
	From
		#tPhraseFromInput
	Where
		ixPhrase > @ixLoopPhrase	

	if @ixLoopPhrase is null break
	--Get the phrase to be searched
	Select
		@sPhrase = sPhraseFromInput+'.',
		@lenPhrase = len(sPhraseFromInput+'.')	
	From
		#tPhraseFromInput		
	Where
		ixPhrase = @ixLoopPhrase				
	
	--reset the loop factor	
	Select
		@ixLoopPhraseText = 0	
	--Break the phrase into words and search them
	While (@ixLoopPhraseText <= @lenPhrase)
	Begin

		Select
			@sCharToBeWord = substring(@sPhrase,@ixLoopPhraseText,1)
		Select
			@unicodeChar = UNICODE(@sCharToBeWord)

		if 	--Chinese UNICODE charts' range (from www.unicode.org)
			((@unicodeChar>=0x4E00) and (@unicodeChar<=0x9FBF))
				or
			((@unicodeChar>=0x3400) and (@unicodeChar<=0x4DBF))
				or
			((@unicodeChar>=0x20000) and (@unicodeChar<=0x2A6DF))
				or
			((@unicodeChar>=0xF900) and (@unicodeChar<=0xFAFF))
				or
			((@unicodeChar>=0x2F800) and (@unicodeChar<=0x2FA1F))
			--Japanese UNICODE charts' range (from www.unicode.org)
				or
			((@unicodeChar>=0x3040) and (@unicodeChar<=0x309F))
				or
			((@unicodeChar>=0x30A0) and (@unicodeChar<=0x30FF))
				or
			((@unicodeChar>=0x31F0) and (@unicodeChar<=0x31FF))
			--Korean UNICODE charts' range (from www.unicode.org)
				or
			((@unicodeChar>=0xAC00) and (@unicodeChar<=0xD7AF))
				or
			((@unicodeChar>=0x1100) and (@unicodeChar<=0x11FF))
				or
			((@unicodeChar>=0x3130) and (@unicodeChar<=0x318F))

		Begin
			--insert CJK character		
			insert into #tWord (ixPhrase,sWord,ixWordPos)	
			Values(@ixLoopPhrase,@sCharToBeWord,@ixLoopPhraseText)
			--see if it's necessary to insert previous @sWord
			if (@sWord is not null)	
			Begin
			--insert the previous @sWordToBeIndex to the table
			insert into #tWord (ixPhrase,sWord,ixWordPos)	
			Values(@ixLoopPhrase,@sWord,@ixLoopPhraseText)
			End
			--reset both
			select 
				@sCharToBeWord = N'',
				@sWord = N''
			
		End
		else if --Numbers and Latins
			((@unicodeChar>=0x0030) and (@unicodeChar<=0x0039))
				or
			((@unicodeChar>=0x0041) and (@unicodeChar<=0x005a))
				or
			((@unicodeChar>=0x0061) and (@unicodeChar<=0x007a))
				or
			((@unicodeChar>=0x00c0) and (@unicodeChar<=0x00ff))
				or
			((@unicodeChar>=0x0100) and (@unicodeChar<=0x017f))
				or
			((@unicodeChar>=0x0180) and (@unicodeChar<=0x0241))
		Begin
			--Build the string 
			select 
				@sWord = @sWord + @sCharToBeWord
		End
		Else -- If it's neither CJKs nor Latins , it must be some kind of breakers(stem)
		Begin
			--See if it's necessary to insert the previous @sWord
			if (@sWord <> N'')	
			Begin
			--Insert the previous @sWordToBeIndex to the table
			insert into #tWord (ixPhrase,sWord,ixWordPos)	
			Values(@ixLoopPhrase,@sWord,@ixLoopPhraseText-len(@sWord))
			End
			--Reset the both
			select 
				@sCharToBeWord = N'',
				@sWord = N''
		End

		select 
			@ixLoopPhraseText = @ixLoopPhraseText + 1		

	End -- end one phrase characters

End -- end phrases

--Remove the blank just in case
delete
from
	#tWord 	
where
	sWord = N''

--Remove the noise words
delete 
	#tWord
From
	#tWord tmp
Where
	sWord in
	(
	Select
		sWordNoise
	From
		trefWordNoise 
	)

--Update the ixWordIndex
Update #tWord
Set
	#tWord.ixWordIndex = indx.ixWordIndex
From
	#tWord
Inner join
	tdatWordIndex indx
on
	#tWord.sWord = indx.sWord

--WordFirstPos (We care about the first word postion of a phrase in this table)
Insert into #tWordFirstPos
(
	ixPhrase,
	ixWordPos
)
Select
	ixPhrase,
	min(ixWordPos)
From
	#tWord
Group by
	ixPhrase

update fst
Set
	fst.ixWordIndex = word.ixWordIndex,
	fst.ixWordSort = word.ixWordSort
From
	#tWordFirstPos fst
inner join
	#tWord word
on
	fst.ixPhrase = word.ixPhrase
and
	fst.ixWordPos = word.ixWordPos

--The first word is NOT involved in the search loop, only the rest words are
Update word
set
	word.ynLoop = 'no'
From
	#tWord word
inner join
	#tWordFirstPos fst
on
	fst.ixPhrase = word.ixPhrase
and
	fst.ixWordPos = word.ixWordPos

------------------------------------
--PART 3: Each phrase leads to a search result set
------------------------------------

------------------------------------
--customize declaration
------------------------------------
Declare 
	--Count how many phrases (each phrase leads to a search result set)
	@cntPhrase int,

	--Deal with how many searched results in one presentation page
	@ixSortPageMin int,
	@ixSortPageMax int,
	@cntEntry int,

	--loop the word in one phrase
	@ixWordLoop int,

	--The word index for the second word,third word(and so on) in one phrase.
	@ixWordIndex int,

	--usually it is 1, unless the first word is a noise word
	@ixWordFirstRelativePos int, 

	--second word, third word(and so on)'s relative position in one phrase	
	@ixWordRelativePos int,

	--the phrase id
	@ixPhrase int


--this table handling paging and performance counter(how many seconds this proc gets results)
create table #tProcResult
(
	maxPageNum int,
	millisec int
)


--The first word of a phrase is very important, it is the start point
--Some records will be removed from this table at every loop.
create table #tSearchFirstWordPos
(
	--this is the primary key for this table
	ixFirstWordKey int IDENTITY (1, 1) NOT NULL,

	ixPhrase int,
	ixFirstWordIndex int,
	ixFirstWordPos int,
	ixArchiveEntry int
)
--this table is a helper table for #tSearchFirstWordPos. 
--It will help removing unneeded ArchiveEntry after all the loops done
create table #tSearchFirstWordPos_GroupBy
(
	ixPhrase int,		
	ixArchiveEntry int
)

--this table is also a helper table. 
--It will help removing unneeded ArchiveEntry after all the loops done
create table #tSearchFirstWordPos_EntryCount
(
	ixArchiveEntry int,
	cntEntry int
)

--this table is the MOST important helper table, and will be cleaned at every loop
--this table is to handle the second/third word (and so on) in a phrase
--if the second/third words (and so on) have the same relative positions as they are in the input phrase,
--we keep the ixFirstWordKey by marking "yes", otherwise we mark "no" to it
create table #tSearchRestWordExpectPos
(
	--to link back to the #tSearchFirstWordPos
	ixFirstWordKey int,
	ixRestWordExpectPos int,
	ixArchiveEntry int,
	ynKeep varchar(3) default 'no'
)


--this table is the search result
create table #tSearchResult
(
	--paging purpose
	ixSort int IDENTITY (1, 1) NOT NULL,
	ixArchiveEntry int,
	ixPhrase int,
	ixFirstWordPos int,
	sStringBrief nvarchar(400)
)

------------------------------------
--customize initialization
------------------------------------

--count how many phrases
Select
	@cntPhrase = count(sPhraseFromInput)
From
	#tPhraseFromInput	

--Insert the first word of each phrase to this table
--Some records will be removed from this table at every loop.
Insert into #tSearchFirstWordPos
(
	ixPhrase,
	ixFirstWordPos,
	ixFirstWordIndex,
	ixArchiveEntry
)
Select
	fst.ixPhrase,
	indx.ixWordStartPos,
	fst.ixWordIndex,
	indx.ixArchiveEntry
From
	#tWordFirstPos fst
inner join
	tdatWordIndex2Text indx
on
	fst.ixWordIndex = indx.ixWordIndex

------------------------------------
--Main Logic
------------------------------------

--loop the input-phrase-word one by one (starts from the second word of the first phrase)
Select
	@ixWordLoop = 0
while(1=1)
Begin
	Select
		@ixWordLoop = min(ixWordSort)
	From
		#tWord
	Where
		ixWordSort > @ixWordLoop
	and
		ynLoop = 'yes'	

	if @ixWordLoop is null break

	--get the second/third (and so on) word information of one phrase
	Select
		@ixWordIndex = ixWordIndex,
		@ixWordRelativePos = ixWordPos,
		@ixPhrase = ixPhrase
	From
		#tWord
	Where
		ixWordSort = @ixWordLoop

	--get the current first word position in the phrase 
	--(In most cases, it is 1 unless the original first word is a noise word and has been removed)
	Select
		@ixWordFirstRelativePos = ixWordPos
	From
		#tWordFirstPos
	Where
		ixPhrase = @ixPhrase 


	--clean up the helper table;
	--this table is to handle the second/third word (and so on) in a phrase;
	--if the second/third words (and so on) have the same relative positions as they are in the input phrase,
	--we keep the ixFirstWordKey by marking "yes", otherwise we mark "no" to it.
	Delete
	From
		#tSearchRestWordExpectPos		
	--Insert the rest table (only current phrase matters)
	Insert into #tSearchRestWordExpectPos
	(
		ixFirstWordKey,
		ixRestWordExpectPos,
		ixArchiveEntry,
		ynKeep
	)
	Select
		ixFirstWordKey,
		--ixFirstWordPos + @ixWordRelativePos - 1,
		ixFirstWordPos + @ixWordRelativePos - @ixWordFirstRelativePos,
		ixArchiveEntry,
		'no'--by default	
	From
		#tSearchFirstWordPos
	Where
		ixPhrase = @ixPhrase
			
	--now we see do we really keep it or not
	Update #tSearchRestWordExpectPos
	Set
		ynKeep = 'yes'
	From
		#tSearchRestWordExpectPos expect
	inner join
		tdatWordIndex2Text indx
	on
		expect.ixRestWordExpectPos = indx.ixWordStartPos
	and
		expect.ixArchiveEntry = indx.ixArchiveEntry
	Where
		indx.ixWordIndex = @ixWordIndex 

	--link back to the first word table and delete things
	Delete #tSearchFirstWordPos 
	From 
		#tSearchFirstWordPos fst
	inner join
		#tSearchRestWordExpectPos rest
	on
		fst.ixFirstWordKey = rest.ixFirstWordKey
	where
		rest.ynKeep = 'no'
	
End	

Select
	@cntPhrase = count(*)
From
	#tPhraseFromInput	

--staging step for the next step 
Insert into #tSearchFirstWordPos_GroupBy
(
	ixPhrase,
	ixArchiveEntry
)
Select
	ixPhrase,
	ixArchiveEntry
From
	#tSearchFirstWordPos
Group by
	ixPhrase,
	ixArchiveEntry
--count ArchiveEntry
Insert into #tSearchFirstWordPos_EntryCount
(
	ixArchiveEntry,
	cntEntry
)
Select
	ixArchiveEntry,
	count(ixArchiveEntry)
From
	#tSearchFirstWordPos_GroupBy
group by
	ixArchiveEntry

--any ArchiveEntry count less than phrase count is not for the final result
Delete #tSearchFirstWordPos
From
	#tSearchFirstWordPos pos
Inner join
	#tSearchFirstWordPos_EntryCount	helper
On
	pos.ixArchiveEntry = helper.ixArchiveEntry 
Where
	helper.cntEntry < @cntPhrase 
----------------------------------------------------
--All the search has done from the above code
--Herebelow is just for the result presentation
----------------------------------------------------
Insert into #tSearchResult
	(
	ixArchiveEntry,
	ixPhrase,
	ixFirstWordPos
	)
Select
	ixArchiveEntry,
	ixPhrase,
	ixFirstWordPos
From
	#tSearchFirstWordPos
Order by
	--this is the time sequence
	ixArchiveEntry,
	--this is the content sequence
	ixFirstWordPos 

---take care of paging
--@ixPage should start from 1, if it's 0, it is an indicator of default latest page
Select 
	@cntEntry = count(ixSort)
From
	#tSearchResult	

--latest (biggest) page
if @cntEntry%@lPageUnit = 0
Begin
	Insert into #tProcResult
	(
		maxPageNum
	)
	values
	(
		@cntEntry/@lPageUnit 
	)
	
End 
Else
Begin
	Insert into #tProcResult
	(
		maxPageNum
	)
	values
	(
		@cntEntry/@lPageUnit + 1
	)
End

--count time 
Update #tProcResult
Set millisec = DATEDIFF(ms,@dtStartTime,getdate())

--ixPage=0 means showing the latest max page
If @ixPage = 0
Begin
	Select
		@ixPage = isnull(maxPageNum,1)
	From
		#tProcResult		
End
--SortMin and SortMax range
Select
	@ixSortPageMin = (@ixPage-1)*@lPageUnit+1
Select
	@ixSortPageMax = @ixSortPageMin+@lPageUnit-1		

--Clean up the paging table
Delete From 
	#tSearchResult
Where
	ixSort < @ixSortPageMin
or
	ixSort > @ixSortPageMax		

/*----debug---
select * from #tWord
select * from #tWordFirstPos
select * from #tSearchFirstWordPos
select * from #tSearchResult
goto procend
*/



--now it's time to handle sStringBrief
Update result 
Set
	sStringBrief =  case 
			when ixFirstWordPos>100 then 
				substring(sEntryText,ixFirstWordPos-100,350)
			else
				substring(sEntryText,0,350)		
			end	
From
	#tSearchResult result
inner join
	tdatArchiveEntry entry
on
	result.ixArchiveEntry = entry.ixArchiveEntry

--export this as a main search result
Select
	tmp.ixArchiveEntry,	
	convert(nvarchar(10),entry.dtCreate,111) as sPostDate,
	tmp.sStringBrief

From
	#tSearchResult tmp
inner join
	tdatArchiveEntry entry
on
	tmp.ixArchiveEntry = entry.ixArchiveEntry
order by
	entry.dtCreate desc


--we still need the maxPageNum
Select
	isnull(maxPageNum,1) as maxPageNum,
	isnull(millisec,0) as millisec
From
	#tProcResult

--We also need to export phrase table so we don't have to redo it from the front end
--sort by length of the phrase
Select
	sPhraseFromInput 	
From
	#tPhraseFromInput
Order by
	len(sPhraseFromInput) desc 	

select * from #tword

------------------------------------
--OutPut
------------------------------------
procend:

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

